#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""
    2016/3/18  WeiYanfeng
    封装线程全局等待标记
"""
import sys
from weberFuncs import PrintTimeMsg, PrintAndSleep


class CGlobalExitFlag:
    WAIT_SECONDS_SYN = 0.01  # 同步等待时长,秒

    def __init__(self):
        self.WAIT_SECONDS_NUM = int(1/self.WAIT_SECONDS_SYN)  # 1秒内的计数
        self.WAIT_SECONDS_LOG = 60*self.WAIT_SECONDS_NUM  # 记录日志计数
        self.dictFlag = dict()  # 通过字典避免global无效问题
        self.dictFlag["gbSigExit"] = False  # Ctrl+C退出信号

    def IsExitFlagTrue(self):
        return self.dictFlag["gbSigExit"]

    def SetExitFlagTrue(self,sMsg):
        self.dictFlag["gbSigExit"] = True
        PrintTimeMsg("%s,gbSigExit=%s" % (sMsg, self.dictFlag["gbSigExit"]))

    def WaitAndCheck(self, sHint, cbCheckFunc=None, iTimeoutSeconds=60):
        # 等待并调用某检查过程
        iWaitCount = 0
        while not self.IsExitFlagTrue():
            PrintAndSleep(self.WAIT_SECONDS_SYN,
                          'WaitAndCheck.%s#%d' % (sHint, iWaitCount),
                          iWaitCount % self.WAIT_SECONDS_LOG == 0)
            if cbCheckFunc:
                lsRet = cbCheckFunc(sHint,iWaitCount)
                if lsRet:
                    return lsRet  # cbFunc 返回非空列表，则直接返回对应值
            else:
                PrintTimeMsg('WaitAndCheck.JustPrint.%s#%d' % (sHint, iWaitCount))
            iWaitCount += 1
            if iWaitCount > self.WAIT_SECONDS_NUM*iTimeoutSeconds:
                return ['WTO','WaitAndCheck.TimeOut']
        return ['WSQ','WaitAndCheck.SigQuit']


def tryMain():
    pass

# -------------------------------------
if __name__ == '__main__':
    tryMain()



#!/usr/local/bin/python
# -*- coding:utf-8 -*-

"""
Created on 2016-7-6
@author: WeiYanfeng

统一定义了错误代码和错误消息模版类 CCSTPError
"""

import sys
from weberFuncs import PrintTimeMsg


class CSTPError:
    """
        集中定义了 CSTP 框架中的错误代码；定义在类中，简化import
    """
    EE = 'EE'
    ES = 'ES'
    CHECK_AUTH_DEFAULT = 10010
    CHECK_AUTH_P2PKIND = 10011
    CHECK_AUTH_HUBID = 10012
    CHECK_AUTH_NO_CIP = 10013
    CHECK_AUTH_CMD_SEQ = 10014
    CHECK_AUTH_KICK_OFFLINE = 10015
    CHECK_AUTH_P2P_PAIRID_1 = 10021
    CHECK_AUTH_P2P_PAIRID_2 = 10022
    CHECK_AUTH_P2P_SUFFIX_PWD = 10023
    CHECK_AUTH_P2P_ACCTID_FMT = 10024
    CHECK_AUTH_P2P_ALREADY_ON = 10025
    P2P_SEND_MSG_NO_SUPPORT_0 = 10100
    P2P_SEND_MSG_CIP_NO_PAIRID = 10101
    P2P_SEND_MSG_CIP_NOT_FOUND = 10102
    P2P_SEND_MSG_SUFFIX_FM_ERR = 10103

    dictErrMsgFmt = {
        CHECK_AUTH_DEFAULT: (ES, 'CheckAuth.DefaultError!',),
        CHECK_AUTH_P2PKIND: (ES, 'sP2PKind={sP2PKind},sAcctId or sAcctPwd error!',),
        CHECK_AUTH_HUBID: (ES, 'sP2PKind={sP2PKind},sHubId={sHubId} not match!',),
        CHECK_AUTH_NO_CIP: (ES, 'dictObjLinkByCIP.get({sClientIPPort})=None!',),
        CHECK_AUTH_CMD_SEQ: (ES, 'Can not call ({cmd0}) after login!',),
        CHECK_AUTH_KICK_OFFLINE: (ES, '({sClientIPPortB}) kicked offline by ({sClientIPPortA}) , sReasonDesc=({sReasonDesc})',),
        CHECK_AUTH_P2P_PAIRID_1: (ES, 'sPairId=({sPairId}) not in lsPairIdAllow!',),
        CHECK_AUTH_P2P_PAIRID_2: (ES, 'sPairId=({sPairId}) NOT found!',),
        CHECK_AUTH_P2P_SUFFIX_PWD: (ES, 'sSuffix=({sSuffix}).password NOT match!',),
        CHECK_AUTH_P2P_ACCTID_FMT: (ES, 'sAcctId=({sAcctId})Format Error!',),
        CHECK_AUTH_P2P_ALREADY_ON: (ES, 'sPairId={sPairId},sSuffix={sSuffix} Already Login on ({sOldIPPort})!',),
        P2P_SEND_MSG_NO_SUPPORT_0: (ES, 'sListSuffixTo=({sListSuffixTo}) include(@) Not supported!',),
        P2P_SEND_MSG_CIP_NO_PAIRID: (ES, 'sClientIPPort=({sClientIPPort}) -> sPairId=NULL!',),
        P2P_SEND_MSG_CIP_NOT_FOUND: (ES, 'sPairId=({sPairId}),sPeerIdTo={sPeerIdTo} -> sClientIPPortTo=NULL!',),
        P2P_SEND_MSG_SUFFIX_FM_ERR: (ES, 'sSuffixFm=({sSuffixFm}), not matches ({sSuffixFmHub})',),
    }

    def __init__(self):
        pass

    @classmethod
    def GenErrorMsgHelpList(cls, sCRLF, sHead, sTail):
        sRet = sHead
        for k,v in cls.dictErrMsgFmt.items():  # 自动按key排序
            sRet += '%s: %s' % (k,','.join(v)) + sCRLF
        sRet += sTail
        return sRet  # str(cls)+'|'+sCRLF


def GenErrorTuple(*args, **kwargs):
    # 生成错误元组，参考 CHubCallbackBasicBase.py 出错出口说明
    # 第一个普通参数是 iErrorNo(sErrorNo) ，第二个可选普通参数是 sErrorDebugMsg
    # 紧接着是零个、一个或多个命名参数，用于格式化生成 sErrorHintMsg
    iErrorNo = 0
    sErrorDebugMsg = ''
    if len(args) >= 1:
        iErrorNo = args[0]
    if iErrorNo == 0:
        sErrorHintMsg = 'iErrorNo=%s=, FirstParamError' % iErrorNo
        PrintTimeMsg('GenErrorTuple.%s!' % sErrorHintMsg)
        return 'EX', str(iErrorNo), sErrorHintMsg, 'Please Check GenErrorTuple() param!'
    if len(args) >= 2:
        sErrorDebugMsg = args[1]
    tupleFmt = CSTPError.dictErrMsgFmt.get(iErrorNo,())
    if len(tupleFmt) < 2:
        sErrorHintMsg = 'dictErrMsgFmt(%s)={%s}=ConfigError' % (iErrorNo, tupleFmt)
        PrintTimeMsg('GenErrorTuple.%s!' % sErrorHintMsg)
        return 'EX', str(iErrorNo), sErrorHintMsg,\
               'Please Check CSTPError.dictErrMsgFmt(%s)!' % iErrorNo
    sErrorHintMsg = tupleFmt[1].format(**kwargs)
    return tupleFmt[0], str(iErrorNo), sErrorHintMsg, sErrorDebugMsg


def GenOkMsgTuple(sOkMsg):
    return 'OK', '0', sOkMsg, ''


def tryMain():
    # print GenErrorTuple(CSTPError.CHECK_AUTH_P2PKIND,'sHint', sP2PKind='sP2PKind')
    PrintTimeMsg( CSTPError.GenErrorMsgHelpList('\n', '', ''))
    pass

# -------------------------------------
if __name__ == '__main__':
    tryMain()



#!/usr/local/bin/python
# -*- coding:utf-8 -*-

"""
Created on 2016-7-1
@author: WeiYanfeng

    定义了 P2PLayout 模式中需要的一些常量
"""

# import sys
# from weberFuncs import PrintTimeMsg
# --------------------------------
# P2PLayout 模式中，内含命令：
# 由服务端发送系统消息；对于接收方来说，则是通知消息
CMD0_P2PLAYOUT_SEND_SYSTEM_MSG = '!P2PLayout.SendSystemMsg'
#  1=sAction       # PeerOnline=上线,PeerOffline=下线
#  2=sPairId
#  3=sSuffix
#  4=sOnlineList   #在线 sSuffix 列表串，逗号分隔
#  5+=其它参数

# 向同一sPairId中的其它Peer发命令请求；对于接收方来说，则是通知消息
CMD0_P2PLAYOUT_SEND_CMD_TOPEER = '!P2PLayout.SendCmdToPeer'
#  1=sSuffixFrom   # Peer消息源，框架程序检查调用方是否填入正确
#  2=sSuffixList   # 逗号分隔的 sSuffix；取值 * ，表示全部其它Peer； *=群发，@=择一
#  3+=其它参数      # WeiYF.20160705 @=择一 这种情况属于负载均衡应用中需要的，这里暂不支持。

# -------------------------------------
# def GetTextPeerIdForP2PLayout(sPairId, sSuffix):
#     return '%s.%s' % (sPairId, sSuffix)


def tryMain():
    pass

# -------------------------------------
if __name__ == '__main__':
    tryMain()



# -*- coding:utf-8 -*-
"""
Created on 2016-5-31
@author: WeiYanfeng

采用类似“纯虚函数”概念来定义 CSTP之Hub服务端 的回调接口基类。

;---------------------------
请求应答模式中，应答消息出口约定，CmdOStr[0] 取值为两个字母。
接收程序通过判断 CmdOStr[0][0]=='O' 来判断该指令执行成功与否。
#CmdOStr[n]成功出口:
0=OK                      ;OK=登录成功,
1+=sSuccessMsg            ;成功时附加消息
#CmdOStr[n]出错出口:
0=E<x>                    ;首字母E表示出错，第2字母x表示错误类型。
                          ;默认EE=普通错误(不区分),ES=系统错误(与应用逻辑的错误，由框架程序处理)
                          ;继续约定: EP=入口参数错误,ET=超时错误,
1=sErrorNo                ;错误代码，编码规则由应用自己定义。建议101+表示应用层面错误类型。
2=sErrorHintMsg           ;出错提示信息，可以展示给最终用户。
3=sErrorDebugMsg          ;出错调试信息，方便开发者调试。

;---------------------------
; 2016-7-1 鉴权类型sP2PKind对应的P2P相关概念，解释如下：
一、典型意义上P2P模式，命名为 P2PCommon 。指的是 sPeerId 是由Peer自己向服务端注册生成的，
Peer自己创建好友关系，然后好友之间就可以直接交互，或者通过群组交互。

二、预分配的P2P模式，命名为 P2PLayout 。 指的是 sPeerId不需要Peer自主注册，需要后台管理方
提前分配 sPairId 和 sSuffix 给Peer，服务端根据sPairId和sSuffix来计算生成sPeerId。
客户端之间是通过 <sSuffix> 相互访问的，不同PairId之间隔绝访问。
sPeerId是计算出来的,仅用于服务端内部查找定位出客户端链接。
    服务端部署需要填入 sHubId, 允许接入的 sPairId 列表（可能存放在数据库中）。
    客户端部署需要填入 sHubId, sPairId, sAcctPwd 和 sSuffix 。
sPairId和sSuffix 编码规则：
    一般要求是字母和数字组合，区分大小写；可以有
    但不可以有如下字符: ~!@#$%^&*()`{}|\ ，但可以有 :;,.+-_

三、共享账号模式，命名为 AcctShare 。指的是在客户端共享账号登录，适用于传统的客户端/服务器模式。
此时服务端的主要工作之将处理客户端的请求并返回应答。不像P2P那样，会在不同客户端之间进行交互。

;---------------------------

客户端登录指令接口描述如下：
[<cstpFuncs.CMD0_CHECK_AUTH>]
@=客户端接入鉴权，同时完成P2P登录。
@1N=1:1                   ;命令字类型;1:1/N
#CmdIStr[n]入口:
1=sP2PKind                ;鉴权类型，取值如下（定义在 mGlobalConst.py 中）：
                          ;     AcctShare=客户端可共享同一个账号鉴权，
                          ;         这种模式不检查 sHubId 是否匹配；而且 ynForceLogin 参数无效；
                          ;     P2PCommon=典型P2P模式，客户端使用自己注册的sPeerId
                          ;         sAcctNo=sPeerId 是提前申请获得的唯一标识
                          ;         sAcctPwd=是提前申请时自定义的访问密码；可修改
                          ;     P2PLayout=预分配P2P模式
                          ;         sAcctNo=sPairId:sSuffix 之间采用英文点号(.)分隔。
                          ;             其中 sPairId 是提前分配的唯一标识
                          ;             其中 sSuffix 是互相间访问后缀标识
                          ;         sAcctPwd=是提前分配的访问密码；可提供修改服务
2=sHubId                  ;服务端标识串；用于客户端识别服务端，避免连错服务器；默认为 @HubId
3=sAcctId                 ;客户端账号标识
4=sAcctPwd                ;客户端账号密码;
5=ynForceLogin            ;YN是否强制登录; N=非强制登录，若检测出对应账号已登录，则返回错误；
                          ;               Y=强制登录，若检测出对应账号已登录，则踢其下线。
6=sClientInfo             ;客户端设备信息，可选
#CmdOStr[n]成功出口
1=sP2PKind                ;鉴权类型
2=sHubId                  ;服务端标识串
3=sAcctId                 ;客户端账号标识
4=sYMDhnsServer           ;服务端当前时间(YYYYMMDD-hhnnss)
5=sServerInfo             ;服务端附加信息，可选
;---------------------------

"""
# import _addpath
from . import _addpath
import sys
from weberFuncs import GetCurrentTime, PrintTimeMsg, PrintAndSleep
from weberFuncs import ClassForAttachAttr, WyfAppendToFile, GetSystemPlatform
from cpyf.cstpFuncs import CMDID_HREAT_BEAT, IsCmdNotify, GetCmdReplyFmRequest
from cpyf.mGlobalConst import P2PKIND_ACCTSHARE
from cpyf.cstpErrorFuncs import CSTPError, GenErrorTuple


class CHubCallbackBasicBase:
    """
        CSTP之Hub服务端 的回调接口, 回调顺序如下：
        HandleServerStart 服务启动
            HandleClientBegin 客户端链接建立
            HandleCheckAuth 客户端鉴权，Login
                HandleRequestCmd 处理客户端发起的请求命令
                HandleCheckReply 检查需要返回客户端的应答数据，包括广播消息
            HandleClientEnd 客户端链接断开，也包括Logout
        HandleServerStop 服务停止
    """
    def __init__(self, sHubId):
        self.sHubId = sHubId
        self.oBind = None  # 监听等待的服务端对象
        # oBind 在 HandleServerStart 中赋值：
        #     oHub = TCmdStringHub服务框架对象标识，可以从中读取一些全局参数
        #   在非空情况下中有如下附加属性：
        #     iNowLinkNum = 当前客户端链接数
        # WeiYF.20160623 使用GetBindAttrValue和GetLinkAttrValue两个函数来获取动态附加属性
        # WeiYF.20160627 以 sClientIPPort 为键值创建一个字典，来保管 oLink 信息。
        self.dictObjLinkByCIP = {}
        # oLink 在 handleClientBegin 中赋值：
        #     sClientIPPort = 客户端的IP地址端口号，冒号分隔
        #   在P2P11模式下，还有
        #     sPeerId = Peer标识
        #     sPairId = Pair标识
        #     sSuffix = Pair后缀
        #   在非空情况下中有如下附加属性：
        #     iRequestCmdNum = 客户端请求计数
        #     iReplyCmdNum = 客户端应答计数
        self.iTestReplyCnt = 0
        self.bQuitLoopFlag = False  # 循环控制变量，由主控程序赋值
        self.sSelfHubId = ''
        self.bVerbosePrintCmdStr = True

        sFNLogClient = 'logClientOnOff.txt'
        self.sFNameLogClient = '/tmp/' + sFNLogClient
        if GetSystemPlatform() == 'Windows':
            # self.sFNameLogClient = 'D:\\' + sFNLogClient
            # WeiYF.20171205 为避免Win7在D:根目录创建文件的权限限制
            # 这里直接采用当前目录的相对路径
            self.sFNameLogClient = r'..\log\%s' % sFNLogClient
        PrintTimeMsg("CHubCallbackBasicBase.sFNameLogClient=(%s)!" % self.sFNameLogClient)

    def __del__(self):
        pass

    def SetCallBackQuitFlag(self, sHint):
        # 设置退出标记
        self.bQuitLoopFlag = True
        PrintTimeMsg("SetCallBackQuitFlag(%s)..." % sHint)

    def SetKickOffFlagTrue(self, sClientIPPort):
        # 设置踢出标记
        # PrintTimeMsg("SetKickOffFlagTrue(%s)..." % sClientIPPort)
        oLink = self.dictObjLinkByCIP.get(sClientIPPort, None)
        if oLink:
            oLink.bKickOff = True
            PrintTimeMsg("SetKickOffFlagTrue(%s)!" % sClientIPPort)

    def GetHubAttrValue(self, sAttrName, sDefValue=''):
        # 根据属性名称，从 oBind.oHub 读取相关属性取值
        if sDefValue == '@':
            sDefValue = '@No %s' % sAttrName
        if self.oBind is None: return sDefValue
        if not hasattr(self.oBind, 'oHub'):
            return sDefValue
        return getattr(self.oBind.oHub,sAttrName,sDefValue)

    def GetLinkAttrValue(self, sClientIPPort, sAttrName, sDefValue=''):
        # 根据属性名称，从 oLink 读取相关属性取值
        oLink = self.dictObjLinkByCIP.get(sClientIPPort,None)
        if sDefValue == '@':
            sDefValue = '@No %s' % sAttrName
        if oLink is None:
            return sDefValue
        return getattr(oLink, sAttrName, sDefValue)

    def HandleServerStart(self, oHub):
        # 处理服务端启动事件
        self.oBind = ClassForAttachAttr()
        self.oBind.oHub = oHub
        self.oBind.iNowLinkNum = 0
        PrintTimeMsg("HandleServerStart(%s,%s)..." % (
            self.sHubId, self.GetHubAttrValue('sServerIPPort', '@')))
        self.bVerbosePrintCmdStr = self.GetHubAttrValue('bVerbosePrintCmdStr', True)

    def HandleServerStop(self):
        # 处理服务端结束事件
        PrintTimeMsg("HandleServerStop(%s,%s)!!!" % (
            self.sHubId, self.GetHubAttrValue('sServerIPPort', '@')))
        self.oBind = None

    def HandleClientBegin(self, sClientIPPort, sock):
        # 处理客户端开启事件
        self.oBind.iNowLinkNum += 1
        oLink = ClassForAttachAttr()
        oLink.sClientIPPort = sClientIPPort
        oLink.sock = sock
        oLink.iNotifyMsgNum = 0
        oLink.iRequestCmdNum = 0
        oLink.iReplyCmdNum = 0
        oLink.bKickOff = False  # 是否被踢下线
        self.dictObjLinkByCIP[sClientIPPort] = oLink
        # WeiYF.20160627 暂时不考虑 sClientIPPort 重复问题
        PrintTimeMsg("HandleClientBegin(%s).iNowLinkNum=%d=" % (
            sClientIPPort, self.oBind.iNowLinkNum))
        WyfAppendToFile(self.sFNameLogClient, '%s=On' % sClientIPPort)

    def HandleClientEnd(self, sClientIPPort):
        # 处理客户端结束事件
        self.oBind.iNowLinkNum -= 1
        del self.dictObjLinkByCIP[sClientIPPort]
        # delattr(self.oLink,'sClientIPPort')
        PrintTimeMsg("HandleClientEnd(%s).iNowLinkNum=%d=" % (
            sClientIPPort, self.oBind.iNowLinkNum))
        WyfAppendToFile(self.sFNameLogClient,'%s=Off' % sClientIPPort)

    def HandleNotifyMsg(self, sClientIPPort, CmdIStr):
        # 处理客户端通知消息，返回bDone=True表示已处理，bDone=False表示未处理
        oLink = self.dictObjLinkByCIP.get(sClientIPPort,None)
        if oLink:
            oLink.iNotifyMsgNum += 1
            PrintTimeMsg("HandleNotifyMsg.Fm(%s)=%s=" % (sClientIPPort, ','.join(CmdIStr)))
            return False
        else:
            PrintTimeMsg("HandleNotifyMsg.Fm(%s).oLink=%s=" % (sClientIPPort, str(oLink)))
            return True

    def HandleRequestCmd(self, sClientIPPort, dwCmdId, CmdIStr):
        # 处理客户端请求命令，返回bDone=True表示已处理，bDone=False表示未处理
        oLink = self.dictObjLinkByCIP.get(sClientIPPort,None)
        if oLink:
            oLink.iRequestCmdNum += 1
            # PrintTimeMsg("HandleRequestCmd.Fm(%s)=%s=" % (sClientIPPort, ','.join(CmdIStr)))
            return False
        else:
            PrintTimeMsg("HandleRequestCmd.Fm(%s).oLink=%s=" % (sClientIPPort, str(oLink)))
            return True

    def HandleCheckAuth(self, sClientIPPort, dwCmdId, CmdIStr):
        # 处理客户端鉴权，返回格式为: (sClientIPPort,dwCmdId,CmdOStr)
        # PrintTimeMsg("HandleCheckAuth.dwCmdId=%d=" % dwCmdId)
        sP2PKind = CmdIStr[1]
        sHubId = CmdIStr[2]
        sAcctId = CmdIStr[3]
        sAcctPwd = CmdIStr[4]
        ynForceLogin = CmdIStr[5]
        sClientInfo = CmdIStr[6]
        if sP2PKind != P2PKIND_ACCTSHARE and sHubId != self.sHubId:
            CmdOStr = GenErrorTuple(CSTPError.CHECK_AUTH_HUBID,
                                    'CHubCallbackBase.HandleCheckAuth',
                                    sP2PKind=sP2PKind, sHubId=sHubId)
        else:
            oLink = self.dictObjLinkByCIP.get(sClientIPPort,None)
            if oLink:
                CmdOStr = self.DoHandleCheckAuth(oLink, dwCmdId, sHubId, sP2PKind,
                                                 sAcctId, sAcctPwd, ynForceLogin, sClientInfo)
            else:
                CmdOStr = GenErrorTuple(CSTPError.CHECK_AUTH_NO_CIP,
                                        'CHubCallbackBase.HandleCheckAuth',
                                        sClientIPPort=sClientIPPort)
        return sClientIPPort, GetCmdReplyFmRequest(dwCmdId), CmdOStr

    def DoHandleCheckAuth(self, oLink, dwCmdId, sHubId, sP2PKind,
                          sAcctId, sAcctPwd, ynForceLogin, sClientInfo):
        # 处理客户端鉴权，返回格式为: CmdOStr
        return GenErrorTuple(CSTPError.CHECK_AUTH_P2PKIND,
                             'CHubCallbackBase.DoHandleCheckAuth',
                             sP2PKind=sP2PKind)

    def HandleCheckAllLinkReply(self):
        # 检查所有链接的应答返还消息（包括通知消息）等，返回格式为: (sClientIPPort,dwCmdId,CmdOStr)
        #   其中，返回 ('',0,[]) 表示没有数据要返回。sClientIPPort取值为*，表示是广播到所有客户端
        (sClientIPPort,dwCmdId,CmdOStr) = self.DoHandleCheckAllLinkReply()
        if sClientIPPort:
            oLink = self.dictObjLinkByCIP.get(sClientIPPort,None)
            if oLink:
                oLink.iReplyCmdNum += 1  # self.GetHubAttrValue('sHubId','@')
                if self.bVerbosePrintCmdStr:
                    PrintTimeMsg("HandleCheckAllLinkReply.Fm(%s).iReplyCmdNum=%s=" % (
                        sClientIPPort, oLink.iReplyCmdNum ))
        return sClientIPPort, dwCmdId, CmdOStr

    def DoHandleCheckAllLinkReply(self):
        # 处理检查所有链接的应答返还消息（包括通知消息）等，返回接口与 HandleCheckAllLinkReply 一致
        return '', 0, []


def tryCHubCallbackBasicBase():
    bhc = CHubCallbackBasicBase('sHubId')  # oObj
    oBind = ClassForAttachAttr()
    bhc.HandleServerStart(oBind)
    print(0, bhc.GetLinkAttrValue('sClientIPPort', 'testAttr'))
    sClientIPPort = 'sClientIP:8888'
    bhc.HandleClientBegin(sClientIPPort, None)
    print(1, bhc.GetLinkAttrValue('sClientIPPort', 'testAttr'))
    bhc.HandleClientEnd(sClientIPPort)
    print(2, bhc.GetLinkAttrValue('sClientIPPort', 'testAttr'))
    bhc.HandleServerStop()


def tryReturnTuple():
    def returnNone():
        return None
    (a, b, c) = returnNone()  # 此时会出异常
    print(a, b, c)

if __name__ == '__main__':
    # tryReturnTuple()
    tryCHubCallbackBasicBase()



#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""
    2016/3/18  WeiYanfeng
    封装操作队列的线程
"""

import sys

from threading import Thread
try:
    from Queue import Queue, Empty
except ImportError:
    from queue import Queue, Empty  # python 3.x

from weberFuncs import PrintTimeMsg, PrintAndSleep, GetCurrentTime
from .CGlobalExitFlag import CGlobalExitFlag


class CQueueObject:
    def __init__(self,sQueueName):
        self.sQueueName = sQueueName
        self.queue = Queue()  # (maxsize = 1000)  # 处理队列，默认容量无限制

    def GetQueueName(self):
        return self.sQueueName

    def GetQueueSize(self):
        return self.queue.qsize()

    def PutToQueue(self, oObject):
        dictObj = dict()
        dictObj['object'] = oObject
        dictObj['ins_time'] = GetCurrentTime()
        self.queue.put(dictObj, block=True)
        return self.queue.qsize()

    def GetFmQueue(self, sThreadName):
        while True:
            try:
                dictObj = self.queue.get(timeout=0.1)  # timeout 0.1s
                self.queue.task_done()
                return dictObj
            except Empty:  # 如果 Queue 是空，就会出异常
                PrintAndSleep(0.1, '%s.GetFmQueue' % sThreadName, False)  # 避免过多日志
                return None


class CQueueThread(Thread):
    def __init__(self, gef, Q, cbDealFunc):
        self.gef = gef
        self.Q = Q
        self.cbDealFunc = cbDealFunc
        self.sQueueName = self.Q.GetQueueName()
        Thread.__init__(self, name='Thread'+self.sQueueName)
        self.setDaemon(True)

    def run(self):  # Overwrite run() method, put what you want the thread do here
        iLoopCnt = 0
        while not self.gef.IsExitFlagTrue():
            iLoopCnt += 1
            try:
                dictObj = self.Q.GetFmQueue(self.getName())
                # iQueueSize = self.Q.GetQueueSize()
                # if iQueueSize or dictObj:
                #     PrintTimeMsg("%s#%d.run.QueueSize=%s,dictObj=(%s)" % (
                #         self.getName(),iLoopCnt,iQueueSize,str(dictObj)))
                if self.cbDealFunc and dictObj:
                    self.cbDealFunc(self.sQueueName, dictObj)
                else:
                    if not self.cbDealFunc:
                        PrintTimeMsg("%s#%d.JustPrint.dictObj=(%s)" % (
                            self.getName(),iLoopCnt, str(dictObj)))
                # PrintAndSleep(5, "_doRun.Just4Debug")
            except Exception as e:
                PrintTimeMsg("CQueueThread.%s.Exception:%s" % (self.getName(), str(e)))
                raise
        PrintTimeMsg("CQueueThread.%s.END.gbSigExit=%s!" % (
                self.getName(),str(self.gef.IsExitFlagTrue())))


class CStartLoopThread(Thread):
    def __init__(self, gef, cbLoopRun):
        self.gef = gef
        self.cbLoopRun = cbLoopRun
        Thread.__init__(self, name='CStartLoopThread')
        self.setDaemon(True)

    def run(self):  # Overwrite run() method, put what you want the thread do here
        iLoopCnt = 0
        while not self.gef.IsExitFlagTrue():
            iLoopCnt += 1
            try:
                if self.cbLoopRun:
                    self.cbLoopRun(iLoopCnt)
                else:
                    PrintAndSleep(1, "%s.Just4Debug" % (self.getName()))
            except Exception as e:
                PrintTimeMsg("CStartLoopThread.Exception:%s" % (str(e)))
                raise
        PrintTimeMsg("CStartLoopThread.END.gbSigExit=%s!" % (str(self.gef.IsExitFlagTrue())))


# -------------------------------------
def tryThreadMain():
    import signal
    gef = CGlobalExitFlag()

    def SetGlobalFlagToQuit(errno):
        global gef
        gef.SetExitFlagTrue("SetGlobalFlagToQuit.errno=%d" % errno)
        sys.exit(errno)

    def sig_handler(signum, frame):
        global gef
        gef.SetExitFlagTrue("receive a signal %d" % signum)

    signal.signal(signal.SIGINT, sig_handler)
    signal.signal(signal.SIGTERM, sig_handler)

    gQueueSend = CQueueObject('Send')
    gQueueSend.PutToQueue(['Cmd0', 'Cmd1', 'Cmd2'])
    PrintAndSleep(1, 'waitThread')
    threadSend = CQueueThread(gef, gQueueSend, None)
    threadSend.start()
    # threadDeal.join() #加上之后就不退出了
    PrintAndSleep(3, 'testThreadMain')

# -------------------------------------
if __name__ == '__main__':
    tryThreadMain()


#!/usr/local/bin/python
# -*- coding:utf-8 -*-

"""
Created on 2016-5-30
@author: WeiYanfeng

CSTP(Command String List Transfer Protocol) 命令串列表传输协议

借助 gevent 微线程实现CSTP异步接入框架 TCmdStringHub.py
设计目标：在支持传统“客户端/服务器(C/S)”架构前提下，扩展支持“客户端到客户端(P2P)”架构。
相关设计细节如下：
1.全程异步消息。并对消息进行分类：
    A.正向请求应答型。由客户端向服务端发起请求，服务端向客户端回送应答。
    B.反向请求应答型。由服务端向客户端发起请求，客户端向服务端回送应答。
    C.消息通知型。由客户端向服务端发起或者反向发起，但无需接收方回送应答。
    D.心跳包型。由客户端或服务端根据自身状况，间隔10分钟或1小时向对方发起；接收方收到后直接回送结果。
2.对请求应答型消息添加唯一标识，并通过多线程封装等实现同步。
3.心跳包参数设计，参见 CSockReadWrite.py
4.客户端连接上之后，需要首先向服务端发登录指令。这里需要说明如下：
    A.客户端账号串(sAcctIdStr)是在P2P架构中作为交互访问的唯一标识；
    B.相同账号的不同设备通过在sAcctIdStr添加后缀实现；
    C.客户端检查密码登录接口描述，参见 CHubCallbackBasicBase.py

---------------------- 依赖包
pip install gevent

"""
# import ._addpath
from . import _addpath  # WeiYF.20180115 在Python3下这种写法是对的，在Python2下也可以
import sys
import gevent
from gevent.queue import Queue
from gevent.server import StreamServer
# from gevent import monkey
# monkey.patch_all()  # WeiYF.20160322 暂时打上补丁

from weberFuncs import GetCurrentTime, PrintTimeMsg, md5, CAppendLogBase
from cpyf.CSockReadWrite import CSockReadWrite
from cpyf.cstpFuncs import IsHeartBeat, IsCmdNotify, GetCmdType, \
    GetCmdRequestFmReply, GetCmdReplyFmRequest  # CMD0_CHECK_AUTH

from cpyf.mGlobalConst import CMD0_CHECK_AUTH, CMD0_KICK_OFFLINE
from cpyf.cstpErrorFuncs import CSTPError, GenErrorTuple


# -------------------------------------
class TCmdStringHub(CAppendLogBase):
    def __init__(self, sHostName4Param, sServerIPPort, oHubCallback,
                 sLogFileName, bVerbosePrintCmdStr=True):
        # CSTP服务端接入框架类 TCmdStringHub
        # sHostName4Param 以主机名为基础的参数访问标识
        # sServerIPPort 服务端守护的 IP:Port ；其中 Port 是整数串，默认是 8888
        #     IP默认取值为 0.0.0.0 表示服务端不绑定网卡
        # oHubCallback Hub处理回调对象
        # sLogFileName 日志文件参考路径，存储在该文件上一级的 log 子目录下
        CAppendLogBase.__init__(self, sLogFileName, 'log')
        self.bVerbosePrintCmdStr = bVerbosePrintCmdStr

        self.sHostName4Param = sHostName4Param
        self.oHubCallback = oHubCallback

        self.sServerIP = '0.0.0.0'
        self.iServerPort = 8888
        lsServer = sServerIPPort.split(':')
        if len(lsServer) >= 2:
            if lsServer[0]:
                self.sServerIP = lsServer[0]
            self.iServerPort = int(lsServer[1])

        PrintTimeMsg('TCmdStringHub(%s:%s).sHubId=%s' % (
            self.sHostName4Param, self.iServerPort, self.oHubCallback.sHubId))

        self.sServerIPPort = '%s:%s' % (self.sServerIP, self.iServerPort)
        self.oHubCallback.HandleServerStart(self)

        # self.bTelnetTest = True
        self.bTelnetTest = False
        self.iSecondsTimeOut = 3600  # 86400 #

        self.iMaxQueueReqNum = 5  # 微线程请求队列消息积压数目
        self.fSecondsFlowCtrl = 0.0001  # 流控休息秒数
        self.fSecondsToSwitch = 0.0000  # 切换休息秒数
        self.fSecondsEchoCtrl = 0.0001  # 应答控制休息秒数

        self.dictLinkQueue = {}  # 当前连接队列字典
        self.queueRequest = Queue()  # 当前请求队列

        self.greenletRequest = None
        self.greenletReply = None

    def __del__(self):
        self.oHubCallback.HandleServerStop()

    def LoopAndWait(self):
        # PrintTimeMsg('LoopAndWait.To join, telnet %s %d' % (self.sServerIP, self.iServerPort))
        self.greenletRequest = gevent.spawn(self.cbLoopHandleQueueRequest)
        self.greenletRequest.start()
        self.greenletReply = gevent.spawn(self.cbLoopHandleQueueReply)
        self.greenletReply.start()

        s = StreamServer((self.sServerIP, self.iServerPort), self.__handleOneConnection)
        s.serve_forever()

    def cbLoopHandleQueueRequest(self):
        while True:
            sClientIPPort,dwCmdId,CmdStr = self.queueRequest.get()
            if IsCmdNotify(dwCmdId):
                self.oHubCallback.HandleNotifyMsg(sClientIPPort,CmdStr)
            else:
                self.oHubCallback.HandleRequestCmd(sClientIPPort,dwCmdId,CmdStr)
            # gevent.sleep(sleepSeconds)

    def cbLoopHandleQueueReply(self):
        while True:
            (sClientIPPort,dwCmdId, CmdOStr) = self.oHubCallback.HandleCheckAllLinkReply()
            if sClientIPPort == '':  # 表示没有数据
                gevent.sleep(self.fSecondsEchoCtrl)
            else:
                if sClientIPPort == '*':  # 广播给所有链接
                    for v in self.dictLinkQueue.values():
                        v.put([dwCmdId, CmdOStr])
                else:
                    v = self.dictLinkQueue.get(sClientIPPort, None)
                    if v:
                        v.put([dwCmdId, CmdOStr])
                    else:
                        sMsg = 'sClientIPPort=%s,iCmdId=%s,CmdOStr=%s' % (
                            sClientIPPort, dwCmdId, str(CmdOStr))
                        PrintTimeMsg("cbLoopHandleQueueReply.IgnoreReply(%s)" % sMsg)
                        self.WyfAppendToFile('HubIgnoreReply', sMsg)
                gevent.sleep(self.fSecondsToSwitch)

    def __handleOneConnection(self, sock, client_addr):
        sClientIPPort = '%s:%s' % (client_addr[0],client_addr[1])
        self.oHubCallback.HandleClientBegin(sClientIPPort,sock)

        PrintTimeMsg("__handleOneConnection.sock=(%s)..." % str(sock))
        oObj = CSockReadWrite(sock,'S')
        oObj.SetObjIPPort(sClientIPPort)
        oObj.bVerbosePrintCmdStr = self.bVerbosePrintCmdStr

        # self.BroadcastToClients('System','## joined from %s.' % (oObj.GetObjIPPort()))
        # self.WyfAppendToFile('GeventClient','%s.SNetClientJoin' % oObj.sClientIPPort)
        oObj.queueForWrite = Queue()
        self.dictLinkQueue[oObj.GetObjIPPort()] = oObj.queueForWrite
        try:
            r = gevent.spawn(self.__handleClientRead, oObj)
            w = gevent.spawn(self.__handleClientWrite, oObj)
            gevent.joinall([r, w])  # 会阻塞在这里
            # PrintTimeMsg("__handleOneConnection.gevent.joinall END!")
        finally:
            # self.BroadcastToClients('System','## %s left the chat.' % oObj.GetObjIPPort())
            self.__hubQuitAndFreeLink(oObj, '__handleOneConnection')
        # self.oHubCallback.HandleClientEnd(sClientIPPort) # 会在 __hubQuitAndFreeLink 中调用

    def __handleClientRead(self, oObj):
        # 处理从链接上读取数据包
        while not oObj.bQuitLoopFlag:
            try:
                sRet, oTuple = oObj.ReadCmdStrFromLink(self.iSecondsTimeOut)  # 15s 调试用
                if sRet == 'OK':
                    (dwCmdId,CmdStr) = oTuple
                    if IsHeartBeat(dwCmdId):
                        oObj.EchoHeartBeatMsg(CmdStr)
                    else:
                        sCmdType = GetCmdType(dwCmdId)
                        if not self.DealSpecialCmdRequest(oObj, dwCmdId, CmdStr):
                            if self.queueRequest.qsize() > self.iMaxQueueReqNum:
                                # WeiYF.20160324 进行流控，消息积压则暂缓
                                gevent.sleep(self.fSecondsFlowCtrl)  # 0.0001
                            self.queueRequest.put([oObj.GetObjIPPort(), dwCmdId, CmdStr])
                elif sRet == 'TimeOut':
                    oObj.SendHeartBeatMsg('0', 'HeartBeat TimeOut From Server')
                else:
                    sErrMsg = oTuple
                    oObj.SetCloseQuitFlag("__handleClientRead.sErrMsg=(%s)" % oTuple)  # 会退出
                # gevent.sleep(0)
            except Exception as e:
                import traceback
                traceback.print_exc()  # WeiYF.20151022 打印异常整个堆栈 这个对于动态加载非常有用
                PrintTimeMsg('__handleClientRead.Exception.e=(%s)' % str(e))
        self.__hubQuitAndFreeLink(oObj, '__handleClientRead.END')

    def __handleClientWrite(self, oObj):
        q = oObj.queueForWrite
        while (not oObj.bQuitLoopFlag) and q:
            dwCmdId,CmdOStr = q.get()
            oObj.WriteCmdStrToLink(dwCmdId, CmdOStr)
            gevent.sleep(self.fSecondsToSwitch)
            self.DealSpecialCmdReply(oObj, dwCmdId, CmdOStr)
        self.__hubQuitAndFreeLink(oObj, '__handleClientWrite.END')

    def __hubQuitAndFreeLink(self, oObj, sHint):
        if not oObj.bQuitLoopFlag:
            oObj.SetCloseQuitFlag(sHint)
            self.oHubCallback.SetCallBackQuitFlag(sHint)
        if oObj.ChkFirstDoQuitEnd():
            sObjIPPort = oObj.GetObjIPPort()
            self.oHubCallback.HandleClientEnd(sObjIPPort)
            del(self.dictLinkQueue[sObjIPPort])
            PrintTimeMsg("__hubQuitAndFreeLink(%s)ForReason=%s" % (sObjIPPort, sHint))

    def DealSpecialCmdRequest(self,oObj,dwCmdId,CmdIStr):
        # 处理特殊命令请求
        # PrintTimeMsg('DealSpecialCmdRequest.CmdIStr=(%s)!' % str(CmdIStr))
        if CmdIStr[0] == CMD0_CHECK_AUTH:  # .startswith('ABLOGIN.ChkPasswd'):
            if oObj.cLoginStatus != 'L':
                CmdOStr = GenErrorTuple(CSTPError.CHECK_AUTH_CMD_SEQ,
                                        'TCmdStringHub.DealSpecialCmdRequest',
                                        cmd0=CMD0_CHECK_AUTH)
                oObj.WriteCmdStrToLink(GetCmdReplyFmRequest(dwCmdId), CmdOStr)
                oObj.SetCloseQuitFlag('DealSpecialCmdRequest.CMD0_CHECK_AUTH')
            else:
                sClientIPPort = oObj.GetObjIPPort()
                (sClientIPPort,dwCmdId,CmdOStr) = self.oHubCallback.HandleCheckAuth(
                    sClientIPPort, dwCmdId, CmdIStr)
                oObj.WriteCmdStrToLink(dwCmdId, CmdOStr)
            return True
        return False

    def DealSpecialCmdReply(self, oObj, dwCmdId, CmdOStr):
        # 处理特殊命令应答
        # PrintTimeMsg('DealSpecialCmdReply.CmdIStr=(%s)!' % str(CmdIStr))
        if CmdOStr[0] == CMD0_KICK_OFFLINE:  # 被踢出
            sClientIPPortA = CmdOStr[1]
            sClientIPPortB = CmdOStr[2]
            sReasonDesc = CmdOStr[3]
            CmdOStr = GenErrorTuple(CSTPError.CHECK_AUTH_KICK_OFFLINE,
                                    'TCmdStringHub.DealSpecialCmdReply',
                                    sClientIPPortA=sClientIPPortA,
                                    sClientIPPortB=sClientIPPortB,
                                    sReasonDesc=sReasonDesc)
            oObj.WriteCmdStrToLink(dwCmdId,CmdOStr)
            oObj.SetCloseQuitFlag('DealSpecialCmdReply.CMD0_KICK_OFFLINE')
            self.oHubCallback.SetKickOffFlagTrue(sClientIPPortB)
            return True
        return False


def StartCmdStringHub(sHostName4Param, sServerIPPort, clsHubCallBack, tupleClsParam,
                      sLogFileName, bVerbosePrintCmdStr=True):
    oHubCallback = clsHubCallBack(*tupleClsParam)
    c = TCmdStringHub(sHostName4Param, sServerIPPort, oHubCallback, sLogFileName, bVerbosePrintCmdStr)
    c.LoopAndWait()


def mainCmdStringHub():
    from hub.CHubCallbackBasicBase import CHubCallbackBasicBase
    from hub.CHubCallbackQueueBase import CHubCallbackQueueBase
    from hub.CHubCallbackP2PLayout import CHubCallbackP2PLayout
    from hub.demoSettingsP2PLayout import gDictP2PLayoutByPairId
    sHostName4Param = 'LocalTest'
    sServerIPPort = '0.0.0.0:8888'
    # sPythonDir = 'D:/WeiYFGitSrc/PythonProject/WxScanServ/ptSNetSck/'
    sPythonDir = 'D:/WeberWork/ArchitectRoot/WeiYF/GitRoot/BitBucket/WeiYFSrcRoot/wxscanservptSNetSck'

    # sHubId = '@HubId'
    sHubId = md5('%s:%s:@HubId' % (sHostName4Param, sServerIPPort))  #
    # sHubId TCmdStringHub的标识，用于一对一P2P模式中，鉴别客户端身份，避免连错主机
    # lsPairIdAllow = ['one', 'two']
    # sDiffKeyTail = ''
    # tupleClsParam = (sHubId,lsPairIdAllow,sDiffKeyTail)
    tupleClsParamP2PLayout = (sHubId, gDictP2PLayoutByPairId,)
    StartCmdStringHub(sHostName4Param, sServerIPPort,
                      # CHubCallbackBase,
                      # CHubCallbackQueueBase,
                      CHubCallbackP2PLayout, tupleClsParamP2PLayout,
                      sPythonDir+'runSNetHubRedis.py')

# -------------------------------------
if __name__ == '__main__':
    mainCmdStringHub()
